#if NOT_LINUX_KERNEL
#include <stdio.h>
#include <string.h>
#endif

#include "ppctypes.h"
#include "sdma.h"
#include "mgt5200.h"

#include "dma_image.h"
#include "bestcomm_api.h"

#ifdef __MWERKS__
__declspec(section ".text") extern const uint32 taskTable;
__declspec(section ".text") extern const uint32 taskTableBytes;
__declspec(section ".text") extern const uint32 taskTableTasks;
__declspec(section ".text") extern const uint32 offsetEntry;
#else
extern const uint32 taskTable;
extern const uint32 taskTableBytes;
extern const uint32 taskTableTasks;
extern const uint32 offsetEntry;
#endif

extern uint8	*MBarGlobal;

typedef struct SCTDT {
	uint32 start;
	uint32 stop;
	uint32 var;
	uint32 fdt;
	uint32 rsvd1;
	uint32 rsvd2;
	uint32 context;
	uint32 litbase;
} SCTDT_T;


/*!
 * \brief	Load BestComm tasks into SRAM.
 * \param	sdma	Base address of the BestComm register set
 *
 * The BestComm tasks must be loaded before any task can be setup,
 * enabled, etc. This might be called as part of a boot sequence before
 * any BestComm drivers are required.
 */
void TasksLoadImage (sdma_regs *sdma)
{
	uint32 i;
	SCTDT_T *tt;

	/* copy task table from source to destination */
	//memcpy ((void *) sdma->taskBar, (void *) &taskTable, (unsigned long) taskTableBytes);
	
	memcpy ((void *)(MBarGlobal + MBAR_SRAM), (void *) &taskTable, (unsigned long) taskTableBytes);
	
	/* adjust addresses in task table */
	for (i=0; i < (uint32) taskTableTasks; i++) {
		tt = (SCTDT_T *) ((MBarGlobal + MBAR_SRAM) + (uint32) offsetEntry + (i * sizeof (SCTDT_T)));
		tt->start 	+= sdma->taskBar;
		tt->stop 	+= sdma->taskBar;
		tt->var		+= sdma->taskBar;
		tt->fdt		= (sdma->taskBar & 0xFFFFFF00) + tt->fdt;
		tt->context	+= sdma->taskBar;
	}
	
	/* initialize task variable pointers */
	init_dma_image ((MBarGlobal + MBAR_SRAM));
}
