#ifndef __TASKSETUP_BDTABLE_H
#define __TASKSETUP_BDTABLE_H 1

/* only need this to pick up MBAR_DMA_FREE define */
#include "dma_image.h"

/*
 * Table of BD rings for all BestComm tasks indexed by task ID.
 *
 *    +-----+------+--------------+    +------+-------+
 * 0: |numBD|numPtr|BDTablePtr ---|--->|status|dataPtr|
 *    +-----+------+--------------+    +------+-------+
 * 1: |numBD|numPtr|BDTablePtr    |    |status|dataPtr|
 *    +-----+------+--------------+    .      .       .
 * 2: |numBD|numPtr|BDTablePtr ---|-+  .      .       .
 *    .            .              . |  .      .       .
 *    .            .              . |  |status|dataPtr|
 *    .            .              . |  +------+-------+
 * 15:|numBD|numPtr|BDTablePtr    | |
 *    +-----+------+--------------+ |
 *                                  |
 *                                  V
 *                                  +------+--------+--------+
 *                                  |status|dataPtr0|dataPtr1|
 *                                  +------+--------+--------+
 *                                  |status|dataPtr0|dataPtr1|
 *                                  .      .        .        .
 *                                  .      .        .        .
 *                                  .      .        .        .
 *                                  |status|dataPtr0|dataPtr1|
 *                                  +------+--------+--------+
 */
typedef struct {
	uint16 numBD;		/* Size of BD ring									*/
	uint8  numPtr;		/* Number of data buffer pointers per BD			*/
	uint8  apiConfig;	/* API configuration flags							*/
	void   *BDTablePtr;	/* Pointer to BD tables, must be cast to TaskBD1_t	*/
						/*   or TaskBD2_t									*/
} TaskBDIdxTable_t;

typedef enum {
	API_CONFIG_NONE		= 0x00,
	API_CONFIG_BD_FLAG	= 0x01
} ApiConfig_t;

extern TaskBDIdxTable_t TaskBDIdxTable[MAX_TASKS];

/*
 * Allocates BD table if needed and updates the BD index table.
 * Do we want to hide this from the C API since it operates on task API?
 */
void TaskSetup_BDTable (volatile uint32 *BasePtr, 
                        volatile uint32 *LastPtr,
                        int TaskNum, uint32 NumBD, uint16 MaxBD,
                        uint8 NumPtr, ApiConfig_t ApiConfig, uint32 Status );

#endif	/* __TASKSETUP_BDTABLE_H */
